import os
import json
import traceback

from datetime import datetime
from logging import getLogger

from django.conf import settings
from django.http import HttpResponse
from django.views.generic.edit import FormView
from django.urls import reverse_lazy

from .constats import column_header, get_fetch_js

logger = getLogger(__name__)


class DjangoSheetFormView(FormView):
    """
    :model:         add model (not required)
    :header:        set header column (not required)
    :empty_row:     set number of empty row post data
                    (N.B. they are initialized when the first data.js is created)

    :HISTORY:       if True make and populate filelog with edit history
    :LOGROOT:       populate with the root path where we are going to load
                    the various static script files
    :SVIL:          if True active debug mode; if settings.DEBUG is True SVIL
                    will be regardless true
    :TIME_UPDATE:   determines how often you want to save resources
                    (eg 10000 = 10 sec)
    :SYNC_DB:       if True all changes will also go to save in the DB
    """
    model: str = None
    header: list = None
    empty_row: int = 10

    HISTORY: bool = False
    LOGROOT: str = None
    SVIL: bool = False
    TIME_UPDATE: int = 10000
    SYNC_DB: bool = False
    DIR_DATA = "data"

    def __init__(self, *args, **kwargs):
        super(DjangoSheetFormView, self).__init__(*args, **kwargs)
        self.filename = "data_1"
        self.jsdata = None
        self.path_filelog = None
        self.filelog = None

        if settings.DEBUG is True:
            self.SVIL = True

        self.jspath = self.mk_jspath()

        if not self.header:
            self.header = list(self.form_class.base_fields.keys())
        else:
            self.header_validator()

    def dispatch(self, *args, **kwargs):
        self.filename = kwargs.get("sheet", "data_1")
        self.jsdata = f"{self.jspath}/{self.filename}.js"

        # init file log paths
        if self.HISTORY:
            self.path_filelog, self.filelog = self.mk_file_log()
        else:
            self.path_filelog, self.filelog = None, None

        # make data_x.js with data model and empty rows
        self.make_data_js(sync_db=self.SYNC_DB)

        # read populate file log
        self.jsheadersheet()

        # added fetch post file
        self.make_fetch_js()
        return super().dispatch(*args, *kwargs)

    def post(self, request, *args, **kwargs):
        res, msg = self.save()
        if res:
            result = {
                "result": {
                    "success": msg,
                },
            }
            json_data = json.dumps(result)
            return HttpResponse(json_data, content_type="application/json")
        else:
            logger.error(msg)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["FOLDER"] = self.DIR_DATA
        context["FILENAME"] = self.filename
        context["SHEETS"] = ",".join(
            [f.replace(".js", "") for f in os.listdir(self.jspath)]
        )
        return context

    def save(self):
        try:
            data_row = json.loads(self.request.body.decode("utf-8"))
        except Exception as ex:
            logger.error(traceback.format_exc())
            return False, ex

        data = []
        for row in data_row.values():
            data.append(row)

        try:
            self.make_data_js(data, self.SYNC_DB)
        except Exception as ex:
            logger.error(traceback.format_exc())
            return False, ex

        if self.HISTORY:
            self.populate_log()

        return True, "ok"

    def header_validator(self) -> None:
        nr_header = len(self.header)
        nr_base_fields = len(self.form_class.base_fields.keys())
        if nr_header != nr_base_fields:
            if nr_header > nr_base_fields:
                self.header = self.header[nr_base_fields]
            if nr_header < nr_base_fields:
                self.header.extend(["" for _ in range(nr_base_fields - nr_header)])

    def mk_jspath(self) -> str:
        jspath = f"{self.LOGROOT}/jsheet/js/{self.DIR_DATA}"
        os.makedirs(jspath, exist_ok=True)
        return jspath

    def mk_file_log(self) -> (str, str):
        path_logs = str(self.LOGROOT) + "/jsheet/"
        os.makedirs(path_logs, exist_ok=True)

        if self.SVIL:
            now_ = datetime.now().strftime("%d%m%Y")
        else:
            now_ = datetime.now().strftime("%d%m%Y_%H%M%S")

        # init filelog
        filelog = path_logs + "datalog_" + now_ + ".log"
        return path_logs, filelog

    def make_data_js(self, data: list = None, sync_db: bool = False) -> None:
        """
        var data = [
            ['50', 'Robe', 'casa', 'carta', '2019-02-12', 'non pagato'], real data value
            ['', '', '', '', '', ''], empty row (set in init)
        ];
        """
        # init datajs
        datajs = "var data = "
        if not data:
            # open data
            datajs += "["
            if sync_db:
                # prepopulate/sync datajs if exist model and data row in model
                datajs += self.prepopulate_datajs()
                datajs += self.get_emtpy_row()
            elif self.filelog and os.path.exists(self.filelog):
                with open(self.filelog) as fn:
                    last_log = fn.readlines()[-1]
                datajs += last_log.split("_GMT__")[1]
                # write empty row
                if not os.path.exists(self.filelog):
                    datajs += self.get_emtpy_row()
            elif os.path.exists(self.jsdata):
                with open(self.jsdata) as fn:
                    datajs += fn.read().replace("var data = [", "").replace("']];", "']")
            else:
                datajs += self.get_emtpy_row()
            datajs += "];"
        else:
            datajs += str(data) + ";"

        if not self.jsdata.split("/")[-1] == 'None.js':
            with open(self.jsdata, "w") as fl:
                fl.write(datajs)

    def prepopulate_datajs(self):
        datajs = ""
        for value in self.model.objects.all():
            list_vl = []
            for key, field in self.form_class.base_fields.items():
                # get value
                vl = getattr(value, key)

                # set field name
                field = field.__class__.__name__
                if field == "CharField":
                    list_vl.append(str(vl))
                if field == "IntegerField":
                    list_vl.append(str(vl))
                if field == "FloatField":
                    list_vl.append(str(vl))
                if field == "DecimalField":
                    list_vl.append(str(vl))
                if field == "DateField":
                    list_vl.append(str(vl.strftime("%Y-%m-%d")))
                if field == "DateTimeField":
                    list_vl.append(str(vl.strftime("%Y-%m-%d")))
                if field == "FileField":
                    list_vl.append(str(vl))
                if field == "TypedChoiceField":
                    choices = getattr(self.model, key).field.choices
                    vl = dict(choices)[vl]
                    list_vl.append(str(vl))
                if field == "ModelChoiceField":
                    list_vl.append(str(vl))
            datajs += str(list_vl) + ","
        return datajs

    def populate_log(self):
        with open(self.jsdata) as fn:
            datalist = fn.read().replace("var data = ", "")

        datalog = datalist.replace("[['", "['").replace("']];", "']")

        mode = "a" if os.path.exists(self.filelog) else "w"
        space_txt = "\n" if mode == "a" else ""
        with open(self.filelog, mode=mode) as fl:
            log_rows = (
                    datetime.now().strftime(space_txt + "%d/%m/%Y_%H:%M:%S_GMT__") + datalog
            )
            fl.write(log_rows)

    def get_emtpy_row(self, nr_row: int = None) -> str:
        if nr_row is None:
            nr_row = self.empty_row

        empty_row = str(["" for _ in range(len(self.header))])
        return ",".join([empty_row for _ in range(nr_row)])

    def jsheadersheet(self):
        datajs = """
			jspreadsheet(document.getElementById('spreadsheet'), {
				data:data, columns: 
		"""
        header = []
        type_header = ""
        for i, field in enumerate(self.form_class.base_fields.values()):
            # set field name
            field_name = field.__class__.__name__
            if field_name == "CharField":
                type_header = column_header("CharField", self.header[i])
            if field_name == "IntegerField":
                type_header = column_header("IntegerField", self.header[i])
            if field_name == "FloatField":
                type_header = column_header("FloatField", self.header[i])
            if field_name == "DecimalField":
                type_header = column_header("DecimalField", self.header[i])
            if field_name == "DateField":
                type_header = column_header("DateField", self.header[i])
            if field_name == "DateTimeField":
                type_header = column_header("DateTimeField", self.header[i])
            if field_name == "FileField":
                type_header = column_header("FileField", self.header[i])
            if field_name in ["ChoiceField", "TypedChoiceField"]:
                source = [x[1] for x in field._choices]
                type_header = column_header(
                    "TypedChoiceField", self.header[i], source
                ).replace("'['", "['").replace("']'", "']")
            if field_name == "ModelChoiceField":
                source = [x.__str__() for x in field._queryset]
                type_header = column_header(
                    "ModelChoiceField", self.header[i], source
                ).replace("'['", "['").replace("']'", "']")
            header.append(type_header)

        datajs += str(header).replace('"', "").replace(",,", ",") + "})"
        jsfile = os.path.join(self.jspath, "..", "header.js")
        with open(jsfile, "w") as fl:
            fl.write(datajs)

    def make_fetch_js(self):
        # get current url and relative path url
        current_url = self.request.resolver_match.view_name
        post_url = reverse_lazy(current_url, kwargs={'sheet': self.filename})

        # make fetch post JS
        fetch_post = get_fetch_js() % (self.TIME_UPDATE, post_url)

        # make and put scripts in fetch_post.js in jspath dir
        jsfile = os.path.join(self.jspath, "..", "fetch_post.js")
        with open(jsfile, "w") as fl:
            fl.write(fetch_post)



import json

from django.conf import settings


def get_type_header(field: str) -> str:
    with open(settings.BASE_PATH_LIBS + "/assets/constats.json") as fn:
        data = json.loads(fn.read())[field]

    res = json.dumps(data)
    for key in data:
        clean_key = key.replace('"', "")
        res = res.replace(f'"{key}"', clean_key)
    return res


def column_header(field: str, value: str, source: list = None) -> str:
    if source is None:
        clm = get_type_header(field) % value
    else:
        clm = get_type_header(field) % (value, source)
    clm = "".join(clm.split()).replace('"', "'")
    return clm


def get_fetch_js() -> str:
    with open(settings.BASE_PATH_LIBS + "/assets/fetch_js.js") as fn:
        val = fn.read()

    return val



from django import template
from django.utils.safestring import mark_safe

register = template.Library()

CSS_STYLE = """
    <style>
        input {
            width:100%;
            margin:0;
            background:transparent;
            border:0px solid #cccccc;
        }
        .input:focus {
            outline: none !important;
            border:1px solid red;
            box-shadow: 0 0 10px #719ECE;
        }
        .input:focus-visible {
            outline: none !important;
            border:1px solid red;
            box-shadow: 0 0 10px #719ECE;
        }
        table tr td {
            border-right:1px solid #cccccc;
            border-bottom:1px solid #cccccc;
        }
        table {
            background: #fff none repeat scroll 0 0;
            border-left: 1px solid #cccccc;
            border-top: 1px solid #cccccc;
        }

        select {
            -webkit-appearance: none;
            -moz-appearance: none;
            appearance: none;
            /* Remove default arrow */
            /* background-image: url(...); */
            /* Add custom arrow */
        }
    </style>
"""


@register.simple_tag
def display_jsheet(data: dict = None):
    if data is None:
        data = {
            "header": [],
        }

    # set thead
    thead = "<thead class='label'><tr><td width='30' class='label'></td>"
    for i, hd_ in enumerate(data["header"]):
        thead += """
            <td id="col-{i}" width="{width}" align="{align}" title="{value}">{value}</td>
        """.format(
            i=i,
            width=hd_.get("width", ""),
            align=hd_.get("align", ""),
            value=hd_.get("value", ""),
        )
    thead += "</tr></thead>"
    # set tbody
    tbody = "<tbody>"
    for i, key_ in enumerate(data["body"]):
        # init tr in tbody
        tbody += "<tr id='row_{0}'>".format(i)
        # set id class of hidden row
        if "empty_" in str(key_):
            id_class = "id='hidden_row_empty_{0}'".format(i)
        else:
            id_class = "id='hidden_row_{0}'".format(i)
        tbody += """
            <td id='clm-{0}' class='label'>{1}<input type='hidden' value='{2}' {3}></td>
        """.format(i, i + 1, key_, id_class)
        td_query = data["body"][key_]
        for td_ in td_query:
            if isinstance(td_, dict):
                tbody += "<td width='{width}' align='{align}'>".format(
                    width=td_.get("width", ""), align=td_.get("align", "")
                )
                input_type = td_.get("input", "")
                initial = td_.get("initial", "")
                # select ModelChoiceField
                if input_type == '__select__':
                    initial = td_.get("initial", "")
                    query = td_.get("query", "")
                    tbody += "<select id='{title}{i}'>".format(i=i, title=td_.get("title", ""))
                    for val_, label in query:
                        if str(initial) == str(val_):
                            target = "selected"
                        else:
                            target = ""
                        tbody += "<option value='{0}' {2}>{1}</option>".format(val_, label, target)
                    tbody += "</select></td>"
                # select TypedChoiceField
                elif input_type == "__choices__":

                    query = td_.get("query", "")
                    tbody += "<select id='Choices{i}'>".format(i=i)
                    for val_, label in query:
                        if str(initial) == str(val_):
                            target = "selected"
                        else:
                            target = ""
                        tbody += "<option value='{0}' {2}>{1}</option>".format(val_, label, target)
                    tbody += "</select></td>"
                else:
                    tbody += """
                        <input type="{input}" value="{value}"></td>
                    """.format(i=i, input=input_type, value=initial)
            else:
                raise
        tbody += "</tr>"
    tbody += "</tbody>"

    js_script = data.get("jsrows", "")
    jsfooter = data["jsfooter"]

    table = """
    {style}
    <div id="sheet">
        <table class='jexcel bossanova-ui' cellpadding='0' cellspacing='0' id='id_jsheet'>
        {thead}
        {tbody}
        </table>
    </div>
    
    {js_script}
    {jsfooter}
    """.format(
        style=CSS_STYLE, thead=thead, tbody=tbody, js_script=js_script, jsfooter=jsfooter,
    )
    return mark_safe(table)



from django.urls import reverse_lazy


class DjangoSheet:
    """
    :param header: header of sheet (th), this param set the lenght of sheet
    :type header: list
    :param jquery: (QuerySet, form_class.base_fields)
    :type jquery: tuple

    :return: [ReturnDescription]
    :rtype: [ReturnType]
    """
    force_clm = False

    def __init__(
            self,
            header,
            jquery,
            colWidths,
            alignWidths,
            empty_row,
            success_url,
            force_clm=force_clm,
    ):
        self.header = header

        if len(jquery) == 1:
            self.form_fields = jquery
        elif len(jquery) == 2:
            self.queryset, self.form_fields = jquery
        else:
            raise

        self.colWidths = colWidths
        self.alignWidths = alignWidths
        self.empty_row = empty_row
        self.success_url = success_url
        self.force_clm = force_clm
        self.set_header = []

    @property
    def len_row(self):
        return len(self.queryset) if self.queryset is not None else 0

    @property
    def len_row_sum(self):
        return self.len_row + int(self.empty_row)

    @property
    def range_len_row(self):
        return range(self.len_row_sum)

    @property
    def len_clm(self):
        return len(self.header)

    @property
    def set_col_width(self):
        colWidths = []
        if not self.colWidths:
            for i in range(self.len_clm):
                colWidths.append(100)
        else:
            if isinstance(self.colWidths, str):
                self.colWidths = int(self.colWidths)
            if isinstance(self.colWidths, int):
                for i in range(self.len_clm):
                    colWidths.append(self.colWidths)
            if isinstance(self.colWidths, list):
                colWidths = self.colWidths
        return colWidths

    @property
    def set_cell(self):
        """
        :return:    {
                        "title": "txt",
                        "initial": "txt",
                        "query": "array",  # only with choices and modelChoices
                        "width": 160,
                        "align": "center",  # left, center, right
                        "input": "text",  # text, number, float, choices, modelChoices
                    }
        :rtype:     dict
        """
        data = []

        for i, label in enumerate(self.form_fields.keys()):
            field = self.form_fields[label]
            field_class = field.__class__.__name__

            # init cell dict
            cell = {
                "title": label,
                "initial": "",
                "width": self.set_col_width[i],
                "align": self.alignWidths,
            }

            # set all types
            if field_class == 'CharField':
                cell["input"] = "text"
            if field_class in ["IntegerField", "FloatField", "DecimalField"]:
                cell["input"] = "text"
            if field_class == 'DateTimeField' or field_class == 'DateField':
                cell["input"] = "datetime-local"
            if field_class == 'FileField':
                cell["input"] = "file"
            if field_class == 'TypedChoiceField':
                cell["input"] = "__choices__"
                if self.queryset:
                    cell["query"] = getattr(self.queryset[0].__class__, label).field.choices
                else:
                    pass
            if field_class == 'ModelChoiceField':
                cell["input"] = "__select__"
                # set default empty select
                cell["query"] = [("0", "None")]
                if len(self.queryset) >= 1:
                    model_ = getattr(self.queryset[0], label).__class__
                    if model_.__name__ != "NoneType":
                        cell["title"] = model_.__name__
                        cell["query"] = [(x.pk, x.__str__()) for x in model_.objects.all()]

            data.append(cell)
        return data

    @property
    def dict_jsheet(self):
        return {
            "class_table": None,
            "setup": {
                "n_clm": self.len_clm,
                "n_row": (self.len_row, self.empty_row),
            },
            "header": self.header_sheet,
            "body": self.init_data,
            "jsrows": self.js_script_row,
            "jsfooter": self.jsrequest_post,
        }

    @property
    def header_sheet(self):
        thead = []
        for i, value in enumerate(self.header):
            header_ = {
                "title": value.capitalize(),
                "value": value,
                "width": self.set_col_width[i],
                "align": self.alignWidths,
            }
            thead.append(header_)
        return thead

    @property
    def init_data(self):
        order_by = list(self.form_fields.keys())

        init_data = {}
        if self.queryset:
            for i, g in enumerate(self.queryset):
                data = []
                # TODO: if form has '__all__' in field, what do you do?
                if isinstance(order_by, list):
                    for j, x in enumerate(order_by):
                        value_dict = self.set_cell[j]
                        value = getattr(g, x)
                        if value:
                            if value_dict["input"] == "__select__":
                                value_dict["initial"] = value.pk
                            else:
                                value_dict["initial"] = value

                        data.append(value_dict)
                init_data[g.pk] = data

        for g in range(self.empty_row):
            init_data[f"empty_{g}"] = self.set_cell

        return init_data

    @property
    def js_script_row(self):
        # TODO: replace static splice with dynamic splice:
        #   get all input select and splice values in row_v
        js_script = ""
        for i in self.range_len_row:
            # init scripts
            js_script += """
                <script type="text/javascript">
                    var clm_len = document.getElementById('id_jsheet').rows[0].cells.length;
            """
            # get number row and init row_v and append value clean
            js_script += """
                    row_%s.addEventListener("input", function (e) {
                        var row_v = [];
                        for (var i = 0; i < 5; i++) {
                            row_v.push(this.getElementsByTagName('input')[i].value)
                        } 
            """ % i
            # insert model and choice value
            js_script += """ 
                        row_v.splice(1, 0, document.getElementById('User{0}').value.toString())
                        row_v.splice(5, 0, document.getElementById('InvoiceReceiptTypology{0}').value)
                        row_v.splice(6, 0, document.getElementById('PaymentMethod{0}').value)
                        row_v.splice(8, 0, document.getElementById('PaymentMethod{0}').value)
            """.format(i)
            # run update_rows to send request POST
            js_script += """
                        update_rows(row_v, %s);
                    });
                </script>
            """ % i
        return js_script

    @property
    def jsrequest_post(self):
        # TODO: add check login required
        return """
        <script src="https://code.jquery.com/jquery-3.5.1.js"
                integrity="sha256-QWo7LDvxbWT2tbbQ97B53yJnYU3WhH/C8ycbRAkjPDc="
                crossorigin="anonymous">
        </script>
        
        <script type="text/javascript">
        function update_rows(row, n_row) {
            $.ajax({
                type: 'POST',
                url: '%s',
                data: {
                    row_value:row.toString(),
                    csrfmiddlewaretoken:$('input[name=csrfmiddlewaretoken]').val()
                },
                success: function (data) {
                    if (document.getElementById('hidden_row_empty_' + n_row)) {
                        document.getElementById('hidden_row_empty_' + n_row).setAttribute('id', 'hidden_row_' + n_row);
                        document.getElementById('hidden_row_' + n_row).value = data.result.success;
                    }
                },
                failure: function (errMsg) {
                    console.log(errMsg);
                }
            })
        }
        </script>
        """ % reverse_lazy(self.success_url)

    @property
    def jsformulas(self):
        return """
            var formulas = {
                "sum"
                /**
                 * Custom method: SUM - Example: =SUM(C1:C10), =SUM(A1,A2,A3)
                 *
                 * @param string formula
                 * @return total
                 */
                : function (formula) {
                    // Get main table id
                    var id = $(this.instance).prop('id')
                    // Total to be returned
                    var total = 0
                    // Check sum type
                    var d = formula.split(/:/)
                    // Which sum to be used
                    if (d.length < 2) {
                        // Explode by comman and sum all columns in the formula
                        d = formula.split(',')
                        $.each($(d), function (k, v) {
                            v = parseInt($('#' + id).jexcel('getValue', v))
                            total += v
                        })
                    } else {
                        t1 = d[0].match(/[a-zA-Z]+/g)
                        t2 = d[1].match(/[a-zA-Z]+/g)
                        // Sum vertical or horizontal
                        if (t1[0] == t2[0]) {
                            // Some all cells in a vertical way
                            co = t1
                            t1 = d[0].match(/[0-9]+/g)
                            t2 = d[1].match(/[0-9]+/g)
                            for (i = t1; i <= t2; i++) {
                                v = parseInt($('#' + id).jexcel('getValue', co + i))
                                total += v
                            }
                        } else {
                            // Som all cells in a horizontal way
                            t1 = $('#' + id).jexcel('id', d[0]).split('-')
                            t2 = $('#' + id).jexcel('id', d[1]).split('-')
            
                            for (i = t1[0]; i <= t2[0]; i++) {
                                v = parseInt($('#' + id).jexcel('getValue',
                                                                i + '-' + t1[1]))
                                total += v
                            }
                        }
                    }
            
                    try {
                        return total
                    } catch (e) {
                        return null
                    }
                }
            }
        """



import os
import glob
import setuptools

__version__ = "0.3.8"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def get_install_requires() -> list:
    """Returns requirements.txt parsed to a list"""
    fname = os.path.join(os.path.dirname(__file__), "requirements.txt")
    targets = []
    if os.path.exists(fname):
        with open(fname, "r") as f:
            targets = f.read().splitlines()
    return targets


setuptools.setup(
    name="django_jsheet",
    version=__version__,
    author="shadMod",
    author_email="support@shadmod.it",
    description="Little tools to render a simple spreadsheet in webpage",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/shadMod/django-jsheet",
    download_url=f"https://github.com/shadMod/django-jsheet/archive/refs/tags/{__version__}.tar.gz",
    project_urls={
        "Documentation": "https://docs.shadmod.it/django_jsheet/index",
        "GitHub": "https://github.com/shadMod/django-jsheet/",
        "Bug Tracker": "https://github.com/shadMod/django-jsheet/issues/",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    packages=[
        "django_jsheet",
        "django_jsheet.src.core",
        "django_jsheet.src.templatetags",
    ],
    install_requires=get_install_requires(),
    data_files=[
        (
            "libs/assets",
            [fn for fn in glob.iglob("django_jsheet/src/core/assets/**/*", recursive=True) if "." in fn],
        ),
        (
            "libs/libs",
            [fn for fn in glob.iglob("django_jsheet/libs/django_jsheet_assets/assets/**/*", recursive=True) if
             "." in fn],
        ),
    ],
    python_requires=">=3.8",
)



from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def display_bottom_sheet(sheets: str, url_name: str, style_cl: str = None, style_id: str = None):
    if style_cl is None:
        style_cl = ""
    if style_id is None:
        style_id = ""

    sheets = sheets.split(",")
    html = []
    for name in sheets:
        _button = f"""
<input type='button' onclick='location.href="/{url_name}/{name}/";'
value='{name}' class='{style_cl}' id='{style_id}' />
        """
        html.append(_button)
    return mark_safe("".join(html))


@register.simple_tag
def display_jsheet(folder: str = None, filename: str = None, domain: str = None):
    if not folder:
        folder = "data"
    if not filename:
        filename = "data_1"
    if not domain:
        domain = ""

    data_links = f"""
<script src="{domain}/media/jsheet/js/{folder}/{filename}.js"></script>
<script src="{domain}/media/jsheet/js/header.js"></script>
<script src="{domain}/media/jsheet/js/fetch_post.js"></script>
    """
    return mark_safe(data_links.strip())
